package com.tfg.andreu.biblioteca;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons
import android.content.Context;
import android.content.Intent;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.bumptech.glide.Glide;

import java.util.ArrayList;

/*Adaptador del recyclerView per al cataleg de Llibres*/
public class AdaptadorCatalegLlibres extends RecyclerView.Adapter<AdaptadorCatalegLlibres.HolderLlibre> {

    private ArrayList<Llibre> llistaLlibres = new ArrayList<Llibre>(); //Contindra la llista de llibres que s'ha obtingut del servidor
    int numItems; //Contindra la grandaria de la llista de llibres obtinguda del servidor


    /*
    * Constructor que rep com a parametre la llista de llibres que es mostraran
    *
    */
    public AdaptadorCatalegLlibres(ArrayList<Llibre> llistaLlibres){
        this.llistaLlibres = llistaLlibres;
        this.numItems = llistaLlibres.size();
    }


    /*
     * Metode que es crida quan es crea un nou ViewHolder el qual contindra la vista per acada item de la llista.
     * Te com a parametre el ViewGroup en el qual esta contingut i un int amb el que es podria canviar el layout.
     */

    @Override
    public HolderLlibre onCreateViewHolder(ViewGroup viewPare, int viewType){
        LayoutInflater inflador = LayoutInflater.from(viewPare.getContext());
        View view = inflador.inflate(R.layout.item_cataleg_llibres, viewPare, false);
        HolderLlibre holderLlibre = new HolderLlibre(view);
        return holderLlibre;
    }

    /*
    * Metode el qual crida el RecyclerView per a mostrar les dades dels items en una posicio determinada.
    * Te com a parametres el viewHolder que ha de ser actualitzat i la posició que ocupa en la llista el item corresponent
    * */
    @Override
    public void onBindViewHolder(HolderLlibre holder, int posicio){
        Llibre llibreActual = llistaLlibres.get(posicio);
        holder.bindejaDades(llibreActual);
    }

    @Override
    public int getItemCount() {
        return numItems;
    }


    /*
    * Classe que defineix el ViewHolder utilitzat per aquest adaptador de RecyclerView
    *
    * */
    class HolderLlibre extends RecyclerView.ViewHolder implements View.OnClickListener {

        Llibre llibre; //Aquesta variable sera utilitzada per a ser passada com a parametre a la nova activitat encarregada de mostrar els detalls del llibre seleccionat
        TextView titol;
        ImageView imatge;
        TextView descripcio;

        /*
        * Constructor per al ViewHolder de llibres, definim un onClickListener per definir el comportament dels clicks a la llista en el metode onClick
        * */
        public HolderLlibre(View itemView){
            super(itemView);
            titol = (TextView) itemView.findViewById(R.id.rv_item_titol);
            imatge = (ImageView) itemView.findViewById(R.id.rv_item_imatge);
            descripcio = (TextView) itemView.findViewById(R.id.rv_item_descripcio);
            itemView.setOnClickListener(this);
        }

        /* Metode que col·loca les dades del del llibre de la posició actual a dintre de les diferents vistes del viewHolder*/
        private void bindejaDades(Llibre llibre){
           this.llibre=llibre;
           titol.setText(llibre.getTitol());
           descripcio.setText(llibre.getDescripcio());

           // Carreguem la imatge fent servir la llibreria Glide, en cas que la imatge no estigui disponible es mostra una imatge d'error
           Context contextImatge = imatge.getContext();
           Glide.with(contextImatge).load(contextImatge.getString(R.string.domini_servidor)+"/imatges/" + llibre.getImatge())
                   .placeholder(R.drawable.img_no_disponible).error(R.drawable.img_no_disponible).into(imatge);
       }

       /**Metode que defineix el comportament del Click en un item.
        * En aquest cas s'obre una activitat nova passant com a parametre el llibre seleccionat en format JSON
        * */
        @Override
        public void onClick(View v) {
            Context context = itemView.getContext();
            Intent nouIntent = new Intent(context, DetallsLlibre.class);
            nouIntent.putExtra("Llibre", llibre.llibreToJson().toString());
            context.startActivity(nouIntent);
        }
    }
}

