package com.tfg.andreu.biblioteca;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.bumptech.glide.Glide;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import org.json.JSONObject;

import java.util.ArrayList;

import cz.msebera.android.httpclient.Header;


/*Adaptador del recyclerView per a la llista de reserves*/
public class AdaptadorLlistaReserves extends RecyclerView.Adapter<AdaptadorLlistaReserves.HolderLlibreReservat> {

    private ArrayList<Reserva> llistaInfoReserves = new ArrayList<Reserva>(); //Contindra la llista de informacio de les reserves que s'ha obtingut del servidor
    int numItems; //Contindra la grandaria de la llista obtinguda del servidor

    /*
   * Constructor que rep com a parametre la llista de reserves que es mostraran
   *
   */
    public AdaptadorLlistaReserves(ArrayList<Reserva> llistaReserves){
        this.llistaInfoReserves = llistaReserves;
        this.numItems = llistaReserves.size();
    }

    /*
     * Metode que es crida quan es crea un nou ViewHolder el qual contindra la vista per acada item de la llista.
     * Te com a parametre el ViewGroup en el qual esta contingut i un int amb el que es podria canviar el layout.
     */

    @Override
    public HolderLlibreReservat onCreateViewHolder(ViewGroup viewPare, int viewType){
        LayoutInflater inflador = LayoutInflater.from(viewPare.getContext());
        View view = inflador.inflate(R.layout.item_llista_reserves, viewPare, false);
        HolderLlibreReservat holderLlibreReservat = new HolderLlibreReservat(view);
        return holderLlibreReservat;
    }

    /*
   * Metode el qual crida el RecyclerView per a mostrar les dades dels items en una posicio determinada.
   * Te com a parametres el viewHolder que ha de ser actualitzat i la posició que ocupa en la llista el item corresponent
   * */
    @Override
    public void onBindViewHolder(HolderLlibreReservat holder, int posicio){
        Reserva infoItemActual = llistaInfoReserves.get(posicio);
        holder.bindejaDades(infoItemActual);
    }

    @Override
    public int getItemCount() {
        return numItems;
    }

    /*
       * Classe que defineix el ViewHolder utilitzat per aquest RecyclerView
       * */
    class HolderLlibreReservat extends RecyclerView.ViewHolder{

        TextView titol;
        ImageView imatge;
        TextView infoDataRecollida;
        TextView infoDataTornada;
        TextView dataRecollida;
        TextView dataTornada;
        Button bt_ampliarPrestec;
        Button bt_activarRecordatori;
        Button bt_detalls_llibre_reservat;
        ProgressDialog dialegProgres;

        /*
        * Constructor per al ViewHolder de reserves
        * */
        public HolderLlibreReservat(View itemView){
            super(itemView);
            titol = (TextView) itemView.findViewById(R.id.rv_llista_reserves_titol);
            imatge = (ImageView) itemView.findViewById(R.id.rv_llista_reserves_imatge);

            infoDataRecollida = (TextView) itemView.findViewById(R.id.rv_llista_reserves_txt_recollida);
            infoDataTornada = (TextView) itemView.findViewById(R.id.rv_llista_reserves_txt_tornada);
            dataRecollida = (TextView) itemView.findViewById(R.id.rv_llista_reserves_recollida);
            dataTornada = (TextView) itemView.findViewById(R.id.rv_llista_reserves_tornada);
            bt_ampliarPrestec = (Button) itemView.findViewById(R.id.bt_ampliar_prestec);
            bt_activarRecordatori = (Button) itemView.findViewById(R.id.bt_activar_recordatori);
            bt_detalls_llibre_reservat = (Button) itemView.findViewById(R.id.bt_detalls_llibre_reservat);
        }

        /* Metode que col·loca les dades de la reserva de la posició actual a dintre de les diferents vistes del viewHolder*/
        void bindejaDades(Reserva info){

            final int idLlibre = info.getIdLlibre(); //S'utilitzara en el onClickListener del boto per ampliar el prestec com a parametre per la activitat de ampliar prestec.
            titol.setText(info.getTitolLlibre());

            /*Si no hi ha una data de recollida establerta s'amaga la vista*/
            if(info.getDataRecollida() == null || info.getDataRecollida().equals("")){
                dataRecollida.setVisibility(View.INVISIBLE);
                infoDataRecollida.setVisibility(View.INVISIBLE);
            }
            else
            {
                dataRecollida.setVisibility(View.VISIBLE);
                infoDataRecollida.setVisibility(View.VISIBLE);
                dataRecollida.setText(info.getDataRecollida());
            }

            dataTornada.setText(info.getDataTornada());




            if (info.getRecordatori() == true){
                bt_activarRecordatori.setText("Desactivar recordatori");

                bt_activarRecordatori.setOnClickListener(new View.OnClickListener() {

                    @Override
                    public void onClick(View v) {
                        Context context = v.getContext();
                        Intent nouIntent = new Intent(context, ActivarRecordatori.class);
                        SharedPreferences sp= context.getSharedPreferences("Login",0);
                        String idUsuari  = sp.getString("idUsuari",null);
                        nouIntent.putExtra("idUsuari", idUsuari);
                        nouIntent.putExtra("idLlibre", idLlibre);
                        nouIntent.putExtra("recordatoriSiNo", false);
                        context.startActivity(nouIntent);
                    }
                });

            }
            else{
                bt_activarRecordatori.setText("Activar recordatori");

                bt_activarRecordatori.setOnClickListener(new View.OnClickListener() {

                    @Override
                    public void onClick(View v) {
                        Context context = v.getContext();
                        Intent nouIntent = new Intent(context, ActivarRecordatori.class);
                        SharedPreferences sp= context.getSharedPreferences("Login",0);
                        String idUsuari  = sp.getString("idUsuari",null);
                        nouIntent.putExtra("idUsuari", idUsuari);
                        nouIntent.putExtra("idLlibre", idLlibre);
                        nouIntent.putExtra("recordatoriSiNo", true);
                        context.startActivity(nouIntent);
                    }
                });
            }

            bt_ampliarPrestec.setText("Ampliar préstec");

            //Si ja s'ha ampliat el temps de termini mes de 3 cops es deshabilita la opció per tal de que un usuari no pugui tenir el llibre reservat durant massa temps
            if (info.getNumAmpliacions()>3){

                bt_ampliarPrestec.setEnabled(false);
            }
            else{
                bt_ampliarPrestec.setOnClickListener(new View.OnClickListener() {

                    @Override
                    public void onClick(View v) {
                        Context context = v.getContext();
                        Intent nouIntent = new Intent(context, AmpliarPrestec.class);
                        nouIntent.putExtra("idLlibre", idLlibre);
                        context.startActivity(nouIntent);
                    }
                });
            }

            bt_detalls_llibre_reservat.setText("Veure detalls");
            bt_detalls_llibre_reservat.setOnClickListener(new View.OnClickListener() {

                @Override
                public void onClick(View v) {

                    /*En cas que el servidor no sigui instantani retornant les dades es mostra un missatge a l'usuari indicant que s'esta treballant en la seva petició*/
                    dialegProgres = new ProgressDialog(v.getContext());
                    dialegProgres.setMessage("Obtenint informació...");
                    dialegProgres.setCancelable(false);
                    RequestParams params = new RequestParams();
                    params.put("idLlibre", idLlibre);
                    invocarServidorDetallsLlibre(params);
                }
            });


            // Carreguem la imatge fent servir la llibreria Glide, en cas que la imatge no estigui disponible es mostra una imatge d'error
            Context contextImatge = imatge.getContext();
            Glide.with(contextImatge).load(contextImatge.getString(R.string.domini_servidor)+"/imatges/" + info.getImatge())
                    .placeholder(R.drawable.img_no_disponible).error(R.drawable.img_no_disponible).into(imatge);
        }

        //Aquest metode s'executa quan l'usuari vol obtenimr els detalls d'un llibre que te a la llista de reserves
        private void invocarServidorDetallsLlibre(RequestParams params){
            final Context context = titol.getContext(); //Obtenim el context de la vista del titol
            dialegProgres.show();

            AsyncHttpClient client = new AsyncHttpClient();
            client.get(context.getString(R.string.domini_servidor)+"/biblioteca/cerca/fescercasimple",params ,new AsyncHttpResponseHandler() {

                /*S'executa el metode onSuccess si la resposta rebuda conte el codi de resposta Http 200*/
                @Override
                public void onSuccess(int statusCode, Header[] headers, byte[] response) {

                    dialegProgres.hide();
                    try {
                        //Si el servidor retorna una resposta buida salta la exception i vol dir que el codi llegit no correspon a cap llibre
                        JSONObject llibreJSON = new JSONObject(new String(response, "UTF-8"));

                        Intent nouIntent = new Intent(context, DetallsLlibre.class);
                        nouIntent.putExtra("Llibre", llibreJSON.toString());
                        context.startActivity(nouIntent);
                    }catch (Exception e){
                        Log.d("Error","Error en la resposta del servidor (El JSON rebut del servidor pot ser invalid?");
                        e.printStackTrace();
                    }

                }

                /*Si la resposta del servidor NO conte el codi Http de resposta 200 s'executa el metode onFailure*/
                @Override
                public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {

                    dialegProgres.hide();

                    //Depenent del codi de resposta d'error indiquem un missatge o un altre

                    if(statusCode == 500){
                        Toast.makeText(context, "500. Error del servidor", Toast.LENGTH_LONG).show();
                    }
                    else if(statusCode == 404){
                        Toast.makeText(context, "404. Recurs no trobat", Toast.LENGTH_LONG).show();
                    }
                    else{
                        Toast.makeText(context, "Dispositiu sense connexió o servidor no disponible", Toast.LENGTH_LONG).show();
                    }
                }
            });
        }
    }
}


