package com.tfg.andreu.biblioteca;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons
import android.Manifest;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;
import org.json.JSONObject;

import java.util.ArrayList;

import cz.msebera.android.httpclient.Header;

//Activitat que s'executa quan l'usuari vol ampliar el temps de prestec d'un llibre

public class AmpliarPrestec extends AppCompatActivity {

    private ProgressDialog dialegProgres;
    private TextView tv_ampliar_prestec;

    //Menu de navegació
    private ListView lv_llista_menu;
    private DrawerLayout drawer_menu_navegacio;
    private ActionBarDrawerToggle drawerToggle;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_ampliar_prestec);
        getSupportActionBar().setTitle("Ampliar prestec");//Indiquem que estem a la activitat per ampliar un prestec
        Bundle parametres = getIntent().getExtras();

        tv_ampliar_prestec = (TextView) findViewById(R.id.tv_ampliar_prestec);

        int idLlibre = parametres.getInt("idLlibre");

        SharedPreferences sp= getSharedPreferences("Login",0);
        String idUsuari  = sp.getString("idUsuari",null);

        //Si no hi ha un usuari logejat al dispositiu error
        if(idUsuari == null || idUsuari.equals("-1")){
            Toast.makeText(getApplicationContext(), "Error. Usuari no ingresat al sistema", Toast.LENGTH_LONG).show();
            finish();
        }
        else
        {
            dialegProgres = new ProgressDialog(AmpliarPrestec.this);
            dialegProgres.setMessage("Ampliant temps de reserva...");
            dialegProgres.setCancelable(false);

            RequestParams params = new RequestParams();
            params.put("idLlibre", idLlibre);
            params.put("idUsuari", idUsuari);
            invocarServidorAmpliarPrestec(params);
        }

    }

    private void invocarServidorAmpliarPrestec(RequestParams params){
        dialegProgres.show();
        AsyncHttpClient client = new AsyncHttpClient();
        client.get(getString(R.string.domini_servidor)+"/biblioteca/ampliarPrestec/ampliar",params ,new AsyncHttpResponseHandler() {

            /*S'executa el metode onSuccess si la resposta rebuda conte el codi de resposta Http 200*/
            @Override
            public void onSuccess(int statusCode, Header[] headers, byte[] response) {
                dialegProgres.hide();


                try {
                    //Obtenim la resposta del servidor
                    JSONObject resultatAmpliarPrestecJSON = new JSONObject(new String(response, "UTF-8"));

                    Boolean resultatAmpliacioPrestec = resultatAmpliarPrestecJSON.getBoolean("resultatAmpliacioOK");
                    int numeroAmpliacions = resultatAmpliarPrestecJSON.getInt("numeroAmpliacions");

                    //Si s'ha pogut realitzar correctament ->
                    if(resultatAmpliacioPrestec == true && numeroAmpliacions <3 && numeroAmpliacions >=0) {
                        tv_ampliar_prestec.setText("Ampliació realitzada correctament");
                    }
                    else
                    {
                        tv_ampliar_prestec.setText("L'ampliació no s'ha pogut realitzar per un dels seguents motius:\n\n -Es pot ampliar el temps de prestec d'un mateix llibre 3 cops com a màxim. \n\n -Hi ha un altre usuari esperant a que el llibre sigui retornat.");
                    }

                }catch (Exception e){
                    Log.d("Error","Error en la resposta del servidor (El JSON rebut del servidor pot ser invalid?");
                    e.printStackTrace();
                }

            }

            /*Si la resposta del servidor NO conte el codi Http de resposta 200 s'executa el metode onFailure*/
            @Override
            public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {

                dialegProgres.hide();
                //Depenent del codi de resposta d'error indiquem un missatge o un altre

                if(statusCode == 500){
                    Toast.makeText(getApplicationContext(), "500. Error del servidor", Toast.LENGTH_LONG).show();

                }
                else if(statusCode == 404){
                    Toast.makeText(getApplicationContext(), "404. Recurs no trobat", Toast.LENGTH_LONG).show();

                }
                else{
                    Toast.makeText(getApplicationContext(), "Dispositiu sense connexió o servidor no disponible", Toast.LENGTH_LONG).show();

                }
            }
        });

    }

    ///////////////////////////////////////
    //CONFIGURACIÓ DEL MENU DE NAVEGACIÓ///
    ///////////////////////////////////////

    //Executem la creació del menu onStart per si un usuari logeja al telefon actualitzar el menu i mostrar la opció d'anar al llistat de llibres en prestec
    @Override
    protected void onStart() {
        super.onStart();

        SharedPreferences sp= getSharedPreferences("Login",0);
        String idUsuari  = sp.getString("idUsuari",null);

        //Si no hi ha un usuari logejat al dispositiu error
        if(idUsuari == null || idUsuari.equals("-1")){
            Toast.makeText(getApplicationContext(), "Error. Usuari no ingresat al sistema", Toast.LENGTH_LONG).show();
            finish();
        }
        //Passem el idUsuari com a parametre ja que no cal accedir a les sharedPreferences dos vegades un cop a onStart i un altre dintre de afegirMenuNavegacio()
        afegirMenuNavegacio(idUsuari); //Metode que configura el menu de navegació

    }

    private void afegirMenuNavegacio(String idUsuari){

        lv_llista_menu = (ListView)findViewById(R.id.lv_llista_menu);
        drawer_menu_navegacio = (DrawerLayout)findViewById(R.id.drawer_menu_navegacio);

        //Afegim les opcions al menu i el click listener

        ArrayList<String> llistaOpcionsMenu = new ArrayList<String>();

        if(idUsuari == null || idUsuari.equals("-1")){
            llistaOpcionsMenu.add("Inici");
            llistaOpcionsMenu.add("Buscador de llibres");
            llistaOpcionsMenu.add("Escanejar codi QR");
        }
        else{
            llistaOpcionsMenu.add("Inici");
            llistaOpcionsMenu.add("Buscador de llibres");
            llistaOpcionsMenu.add("Escanejar codi QR");
            llistaOpcionsMenu.add("Llibres en prestec");
        }

        lv_llista_menu.setAdapter(new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, llistaOpcionsMenu));

        lv_llista_menu.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(id==0){
                    //Si seleccionem la opció per anar a l'inici
                    startActivity(new Intent(AmpliarPrestec.this,Home.class));
                    finish();//Eliminem la activitat per  no permetre tornar a aquesta activitat amb el boto cap endarrere
                }
                else if(id==1){
                    //Si seleccionem la opció per anar al buscador de llibres
                    startActivity(new Intent(AmpliarPrestec.this,BuscadorLlibres.class));
                    finish();//Eliminem la activitat per  no permetre tornar a aquesta activitat amb el boto cap endarrere
                }
                else if(id==2){
                    //Si seleccionem la opció per escanejar un codi QR

                    //Comprovem que tinguem permisos per accedir a la camera, si no els tenim els demanem
                    if (ContextCompat.checkSelfPermission(AmpliarPrestec.this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {

                        ActivityCompat.requestPermissions(AmpliarPrestec.this, new String[]{Manifest.permission.CAMERA},1);
                    }
                    //Si ja tenim permisos anem directament al lector QR
                    else{
                        startActivity(new Intent(AmpliarPrestec.this,InstruccionsQR.class));
                        finish();//Eliminem la activitat per  no permetre tornar a aquesta activitat amb el boto cap endarrere
                    }
                }
                else if(id==3){
                    //Si seleccionem la opció per mostrar els llibres en prestec de l'usuari
                    startActivity(new Intent(AmpliarPrestec.this,LlistaReserves.class));
                    finish();//Eliminem la activitat per  no permetre tornar a aquesta activitat amb el boto cap endarrere
                }
            }
        });

        drawerToggle = new ActionBarDrawerToggle(this, drawer_menu_navegacio, R.string.menu_navegacio_obert, R.string.menu_navegacio_tancat) {

            /** Quan el menu esta obert per complert canviem el titol de la activitat */
            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                getSupportActionBar().setTitle("Navegador");
                invalidateOptionsMenu();
            }

            /** Quan el menu esta tancat per complert canviem el titol de la activitat */
            public void onDrawerClosed(View view) {
                super.onDrawerClosed(view);
                getSupportActionBar().setTitle("Ampliar prestec");
                invalidateOptionsMenu();
            }
        };

        drawerToggle.setDrawerIndicatorEnabled(true);
        drawer_menu_navegacio.addDrawerListener(drawerToggle);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeButtonEnabled(true);
    }

    //Per a evitar que el menu no estigui sincronitzat cridrem syncState
    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        drawerToggle.syncState();
    }

    //Per a evitar que el menu no estigui sincronitzat quan per exemple passem de orientacio vertical a horitzontal cridem onConfigurationChanged
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        drawerToggle.onConfigurationChanged(newConfig);
    }

    //Metodes que crea el menu de la part superior dreta, on es pot accedir a fer el login/logout
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_usuari, menu);

        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int itemSeleccionat = item.getItemId();

        if(itemSeleccionat == R.id.menu_detalls_usuari){

            SharedPreferences sp= getSharedPreferences("Login",0);
            String idUsuari  = sp.getString("idUsuari",null);

            if(idUsuari == null || idUsuari.equals("-1")){
                startActivity(new Intent(AmpliarPrestec.this,Login.class));
            }
            else{
                startActivity(new Intent(AmpliarPrestec.this,InfoUsuari.class));
            }
            return true;
        }
        if (drawerToggle.onOptionsItemSelected(item)) {
            return true;
        }
        return super.onOptionsItemSelected(item);

    }


    //Metode que s'executa quan l'usuari respon a la otorgació o denegació de permisos
    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case 1: {
                //Si l'usuari ha donat permisos obrim el lector QR
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

                    startActivity(new Intent(AmpliarPrestec.this,InstruccionsQR.class));

                } else {
                    //En cas contrari mostrem un missatge informant que la opció no esta habilitada
                    Toast.makeText(getApplicationContext(), "S'ha denegat el permis per accedir a la camera", Toast.LENGTH_LONG).show();
                }
            }
        }
    }

}
