package com.tfg.andreu.biblioteca;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons
import android.Manifest;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.Toast;

import java.util.ArrayList;

//Activitat que s'encarrega de recollir quines son les paraules clau i el tema dels llibres que vol buscar l'usuari

public class BuscadorLlibres extends AppCompatActivity {

    private Spinner llista_temes; //Llista desplegable amb els temes que es poden cercar
    private Button bt_fes_cerca; //Boto per realitzar la cerca
    private EditText paraulesClau; //Paraules clau introduides per l'usuari

    //Menu de navegació
    private ListView lv_llista_menu;
    private DrawerLayout drawer_menu_navegacio;
    private ActionBarDrawerToggle drawerToggle;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_buscador_llibres);
        getSupportActionBar().setTitle("Buscador de llibres");//Indiquem que estem a la activitat per a buscar llibres

        //Creem la llista desplegable de temes per a cercar i especifiquem quin tipus de layout ha de tenir
        llista_temes = (Spinner) findViewById(R.id.llista_temes);
        ArrayAdapter<CharSequence> adaptador = ArrayAdapter.createFromResource(this, R.array.temes_llibres, android.R.layout.simple_spinner_item);
        adaptador.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        llista_temes.setAdapter(adaptador);


        paraulesClau = (EditText) findViewById(R.id.paraulesClau);
        bt_fes_cerca = (Button) findViewById(R.id.bt_fes_cerca);

        /*Definim el comportament del click al boto de cerca*/
        bt_fes_cerca.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                String stringParaulesClau = paraulesClau.getText().toString();
                String stringTema = llista_temes.getSelectedItem().toString();

                //Si no ens introdueixen almenys un tema o be unes paraules clau, error->
                if(stringEsNull(stringParaulesClau) && stringEsNull(stringTema)){
                    Toast.makeText(getApplicationContext(), "Introdueix unes paraules clau o un tema!", Toast.LENGTH_LONG).show();
                }
                else {
                    //Si l'usuari introdueix dades amb les que fer una cerca, executem l'activitat del cataleg passant com a parametre les paraules clau i/o el tema seleccionat
                    Intent intent = new Intent(BuscadorLlibres.this, CatalegLlibres.class);
                    Bundle parametres = new Bundle();
                    parametres.putString("paraulesClau", stringParaulesClau);
                    parametres.putString("tema", stringTema);

                    intent.putExtras(parametres);
                    startActivity(intent);
                }

            }
        });


    }

    /*Metode auxiliar que comprova si un string es null o esta buit*/
    private boolean stringEsNull (String text){
        if (text != null && text.trim().length() > 0){
            return false;
        }
        else{
            return true;
        }
    }



    ///////////////////////////////////////
    //CONFIGURACIÓ DEL MENU DE NAVEGACIÓ///
    ///////////////////////////////////////

    //Executem la creació del menu onStart per si un usuari logeja al telefon actualitzar el menu i mostrar la opció d'anar al llistat de llibres en prestec
    @Override
    protected void onStart() {
        super.onStart();
        afegirMenuNavegacio(); //Metode que configura el menu de navegació
    }

    private void afegirMenuNavegacio(){

        lv_llista_menu = (ListView)findViewById(R.id.lv_llista_menu);
        drawer_menu_navegacio = (DrawerLayout)findViewById(R.id.drawer_menu_navegacio);

        //Afegim les opcions al menu i el click listener

        SharedPreferences sp= getSharedPreferences("Login",0);
        String idUsuari  = sp.getString("idUsuari",null);

        ArrayList<String> llistaOpcionsMenu = new ArrayList<String>();

        if(idUsuari == null || idUsuari.equals("-1")){
            llistaOpcionsMenu.add("Inici");
           // llistaOpcionsMenu.add("Buscador de llibres");
            llistaOpcionsMenu.add("Escanejar codi QR");
        }
        else{
            llistaOpcionsMenu.add("Inici");
          //  llistaOpcionsMenu.add("Buscador de llibres");
            llistaOpcionsMenu.add("Escanejar codi QR");
            llistaOpcionsMenu.add("Llibres en prestec");
        }

        lv_llista_menu.setAdapter(new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, llistaOpcionsMenu));

        lv_llista_menu.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(id==0){
                    //Si seleccionem la opció per anar a l'inici
                    startActivity(new Intent(BuscadorLlibres.this,Home.class));
                }
                else if(id==1){
                    //Si seleccionem la opció per escanejar un codi QR

                    //Comprovem que tinguem permisos per accedir a la camera, si no els tenim els demanem
                    if (ContextCompat.checkSelfPermission(BuscadorLlibres.this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {

                        ActivityCompat.requestPermissions(BuscadorLlibres.this, new String[]{Manifest.permission.CAMERA},1);
                    }
                    //Si ja tenim permisos anem directament al lector QR
                    else{
                        startActivity(new Intent(BuscadorLlibres.this,InstruccionsQR.class));
                    }
                }
                else if(id==2){
                    //Si seleccionem la opció per mostrar els llibres en prestec de l'usuari
                    startActivity(new Intent(BuscadorLlibres.this,LlistaReserves.class));
                }
            }
        });

        drawerToggle = new ActionBarDrawerToggle(this, drawer_menu_navegacio, R.string.menu_navegacio_obert, R.string.menu_navegacio_tancat) {

            /** Quan el menu esta obert per complert canviem el titol de la activitat */
            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                getSupportActionBar().setTitle("Navegador");
                invalidateOptionsMenu();
            }

            /** Quan el menu esta tancat per complert canviem el titol de la activitat */
            public void onDrawerClosed(View view) {
                super.onDrawerClosed(view);
                getSupportActionBar().setTitle("Buscador de llibres");
                invalidateOptionsMenu();
            }
        };

        drawerToggle.setDrawerIndicatorEnabled(true);
        drawer_menu_navegacio.addDrawerListener(drawerToggle);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeButtonEnabled(true);
    }

    //Per a evitar que el menu no estigui sincronitzat cridrem syncState
    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        drawerToggle.syncState();
    }

    //Per a evitar que el menu no estigui sincronitzat quan per exemple passem de orientacio vertical a horitzontal cridem onConfigurationChanged
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        drawerToggle.onConfigurationChanged(newConfig);
    }

    //Metodes que crea el menu de la part superior dreta, on es pot accedir a fer el login/logout
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_usuari, menu);

        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int itemSeleccionat = item.getItemId();

        if(itemSeleccionat == R.id.menu_detalls_usuari){

            SharedPreferences sp= getSharedPreferences("Login",0);
            String idUsuari  = sp.getString("idUsuari",null);

            if(idUsuari == null || idUsuari.equals("-1")){
                startActivity(new Intent(BuscadorLlibres.this,Login.class));
            }
            else{
                startActivity(new Intent(BuscadorLlibres.this,InfoUsuari.class));
            }
            return true;
        }
        if (drawerToggle.onOptionsItemSelected(item)) {
            return true;
        }
        return super.onOptionsItemSelected(item);

    }


    //Metode que s'executa quan l'usuari respon a la otorgació o denegació de permisos
    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case 1: {
                //Si l'usuari ha donat permisos obrim el lector QR
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

                    startActivity(new Intent(BuscadorLlibres.this,InstruccionsQR.class));

                } else {
                    //En cas contrari mostrem un missatge informant que la opció no esta habilitada
                    Toast.makeText(getApplicationContext(), "S'ha denegat el permis per accedir a la camera", Toast.LENGTH_LONG).show();
                }
            }
        }
    }

}
