package com.tfg.andreu.biblioteca;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons
import android.Manifest;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.bumptech.glide.Glide;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;

import cz.msebera.android.httpclient.Header;

//Activitat que mostra la informació detallada d'un llibre en concret
public class DetallsLlibre extends AppCompatActivity {

    private Llibre objLlibre;
    private TextView tv_titol;
    private ImageView tv_imatge;
    private TextView tv_disponibilitat;
    private Button bt_reservar;
    private TextView tv_descripcio;
    private TextView tv_autor;
    private TextView tv_data_publicacio;
    private ProgressDialog dialegProgres;
    private String paramDataReserva;

    //Menu de navegació
    private ListView lv_llista_menu;
    private DrawerLayout drawer_menu_navegacio;
    private ActionBarDrawerToggle drawerToggle;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_detalls_llibre);
        getSupportActionBar().setTitle("Detalls del llibre");//Indiquem que estem a la activitat per mostrar els detalls d'un llibre

        /*En cas que el servidor no sigui instantani retornant les dades es mostra un missatge a l'usuari indicant que s'esta treballant en la seva petició*/
        dialegProgres = new ProgressDialog(DetallsLlibre.this);
        dialegProgres.setMessage("Obtenint informació...");
        dialegProgres.setCancelable(false);

        Bundle parametres = getIntent().getExtras();
        String llibrePasatPerParametre = "";


        if(parametres != null) {
            tv_titol = (TextView) findViewById(R.id.detalls_llibre_titol);
            tv_imatge = (ImageView) findViewById(R.id.detalls_llibre_imatge);
            tv_disponibilitat = (TextView) findViewById(R.id.detalls_llibre_disponibilitat);
            bt_reservar = (Button) findViewById(R.id.detalls_llibre_bt_reservar);
            tv_descripcio = (TextView) findViewById(R.id.detalls_llibre_descripcio);
            tv_autor = (TextView) findViewById(R.id.detalls_llibre_autor);
            tv_data_publicacio = (TextView) findViewById(R.id.detalls_llibre_data);

           llibrePasatPerParametre = parametres.getString("Llibre");

            try {
                objLlibre = new Llibre(new JSONObject(llibrePasatPerParametre));

                tv_titol.append(objLlibre.getTitol());

                tv_descripcio.append(objLlibre.getDescripcio());
                tv_autor.append(objLlibre.getAutor());
                tv_data_publicacio.append(objLlibre.getDataPublicacio());

                Context contextImatge = tv_imatge.getContext();
                Glide.with(contextImatge).load(contextImatge.getString(R.string.domini_servidor)+"/imatges/" + objLlibre.getImatge())
                    .placeholder(R.drawable.img_no_disponible).error(R.drawable.img_no_disponible).into(tv_imatge);

                //Definim el comportamen del boto per a reservar un llibre
                bt_reservar.setOnClickListener(new View.OnClickListener() {

                    @Override
                    public void onClick(View v) {

                        SharedPreferences sp= getSharedPreferences("Login",0);
                        String idUsuari  = sp.getString("idUsuari",null);

                        //Si no hi ha un usuari logejat al sistema, al apretar el boto redirigim a la pàgian de login
                        if(idUsuari == null || idUsuari.equals("-1")){

                            startActivity(new Intent(DetallsLlibre.this,Login.class));
                        }
                        //Si hi ha un usuari logejat al sistema procedim a reservar el llibre
                        else{

                            Intent nouIntent = new Intent(DetallsLlibre.this,FerReserva.class);
                            nouIntent.putExtra("DataMaximaReserva",paramDataReserva);
                            nouIntent.putExtra("idLlibre",Integer.toString(objLlibre.getId()));
                            startActivity(nouIntent);
                        }
                    }
                });
            } catch (JSONException e) {
                Toast.makeText(getApplicationContext(), "Error amb la informacio del llibre", Toast.LENGTH_LONG).show();
                Log.d("Error","Error amb els parametres rebuts del llibre");
                e.printStackTrace();
                finish();
            }
        }
        else{
            Toast.makeText(getApplicationContext(), "Error amb la informacio del llibre", Toast.LENGTH_LONG).show();
            Log.d("Error","Error amb els parametres rebuts del llibre");
            finish();
        }
    }

    /*La petició per obtenir la informació de la reserva del llibre s'executa onStart() per tal de que s'actualitzi al tornar a aquesta activitat si un usuari logeja a l'aplicacio*/
    //Executem la creació del menu onStart per si un usuari logeja al telefon actualitzar el menu i mostrar la opció d'anar al llistat de llibres en prestec
    @Override
    protected void onStart() {
        super.onStart();
        afegirMenuNavegacio(); //Metode que configura el menu de navegació
        RequestParams params = new RequestParams();
        params.put("idLlibre", objLlibre.getId());
        invocarServidorInfoLlibre(params);

    }

    /*Aquest metode s'encarrega d'obtenir la informacio de reserva del llibre i actualitzar la interficie en cas necessari*/
    private void invocarServidorInfoLlibre(RequestParams params){

        dialegProgres.show();

        AsyncHttpClient client = new AsyncHttpClient();
        client.get(getString(R.string.domini_servidor)+"/biblioteca/infoReserva/info",params ,new AsyncHttpResponseHandler() {

            /*S'executa el metode onSuccess si la resposta rebuda conte el codi de resposta Http 200*/
            @Override
            public void onSuccess(int statusCode, Header[] headers, byte[] response) {

                dialegProgres.hide();
                tv_disponibilitat.setTextColor(Color.BLACK);
                bt_reservar.setVisibility(View.VISIBLE);

                try {

                    JSONObject infoLlibre = new JSONObject(new String(response, "UTF-8")); //Obtenim la resposta del servidor
                    String dataReserva = infoLlibre.getString("dataReserva"); //La resposta conte la ultima data de la cua de reserves
                    paramDataReserva = dataReserva;
                    JSONArray llistaIdUsuaris =infoLlibre.getJSONArray("llistaIdUsuaris"); //Juntament amb els id de tots els usuaris que tenen el llibre reservat


                    SharedPreferences sp= getSharedPreferences("Login",0);
                    String idUsuariLogejat  = sp.getString("idUsuari",null); //Obtenim el id del usuari logejat al telefon

                    //Si NO hi ha cap usuari logejat al telefon ->
                    if(idUsuariLogejat == null || idUsuariLogejat.equals("-1")) {
                        //i el llibre no te cap reserva ->
                        if (dataReserva.equals("") || dataReserva == null){
                            tv_disponibilitat.setText("El llibre esta disponible");
                        }
                        //En canvi si el llibre te alguna reserva mostrem la data en la que estara disponible
                        else{
                            tv_disponibilitat.setText("El llibre no esta disponible fins al: \n"+dataReserva);
                            tv_disponibilitat.setTextColor(Color.RED);
                        }
                    }
                    //Si hi ha un usuari logejat al telefon ->
                    else{
                        //Pero el llibre no te cap reserva ->
                        if (dataReserva.equals("") || dataReserva == null){
                            tv_disponibilitat.setText("El llibre esta disponible");
                        }
                        //Si el llibre si te alguna reserva, comprovem si una d'elles es la del propi usuari logejat
                        else if(usuariLogejatJaReservat(llistaIdUsuaris, idUsuariLogejat)) {
                            tv_disponibilitat.setText("Ja tens reservat aquest llibre");
                            bt_reservar.setVisibility(View.INVISIBLE);
                        }
                        //En cas que l'usuari logejat no tingui cap reserva d'aquest llibre ->
                        else{
                            tv_disponibilitat.setText("El llibre no esta disponible fins al: \n"+dataReserva);
                            tv_disponibilitat.setTextColor(Color.RED);
                        }

                    }

                }catch (Exception e){
                    Log.d("Error","Error en la resposta del servidor (El JSON rebut del servidor pot ser invalid?");
                    e.printStackTrace();
                }

            }

            /*Si la resposta del servidor NO conte el codi Http de resposta 200 s'executa el metode onFailure*/
            @Override
            public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {

                dialegProgres.hide();
                tv_disponibilitat.append("No es pot obtenir les dades de reserva");
                bt_reservar.setVisibility(View.INVISIBLE);
                //Depenent del codi de resposta d'error indiquem un missatge o un altre

                if(statusCode == 500){
                    Toast.makeText(getApplicationContext(), "500. Error del servidor", Toast.LENGTH_LONG).show();
                }
                else if(statusCode == 404){
                    Toast.makeText(getApplicationContext(), "404. Recurs no trobat", Toast.LENGTH_LONG).show();
                }
                else{
                    Toast.makeText(getApplicationContext(), "Dispositiu sense connexió o servidor no disponible", Toast.LENGTH_LONG).show();
                }
            }
        });

    }

    //Aquest metode comprova si l'usuari logejat al telefon ja te el llibre reservat
    private Boolean usuariLogejatJaReservat (JSONArray llistaIdUsuaris, String idUsuariLogejat) throws JSONException{
        Boolean ret = false;

            for (int i =0; i<llistaIdUsuaris.length();i++){
                if (llistaIdUsuaris.get(i).equals(idUsuariLogejat)){
                    ret = true;
                    break;
                }
            }
        return ret;
    }


    ///////////////////////////////////////
    //CONFIGURACIÓ DEL MENU DE NAVEGACIÓ///
    ///////////////////////////////////////

    private void afegirMenuNavegacio(){

        lv_llista_menu = (ListView)findViewById(R.id.lv_llista_menu);
        drawer_menu_navegacio = (DrawerLayout)findViewById(R.id.drawer_menu_navegacio);

        //Afegim les opcions al menu i el click listener

        SharedPreferences sp= getSharedPreferences("Login",0);
        String idUsuari  = sp.getString("idUsuari",null);

        ArrayList<String> llistaOpcionsMenu = new ArrayList<String>();

        if(idUsuari == null || idUsuari.equals("-1")){
            llistaOpcionsMenu.add("Inici");
            llistaOpcionsMenu.add("Buscador de llibres");
            llistaOpcionsMenu.add("Escanejar codi QR");
        }
        else{
            llistaOpcionsMenu.add("Inici");
            llistaOpcionsMenu.add("Buscador de llibres");
            llistaOpcionsMenu.add("Escanejar codi QR");
            llistaOpcionsMenu.add("Llibres en prestec");
        }

        lv_llista_menu.setAdapter(new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, llistaOpcionsMenu));

        lv_llista_menu.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(id==0){
                    //Si seleccionem la opció per anar a l'inici
                    startActivity(new Intent(DetallsLlibre.this,Home.class));
                }

                else if(id==1){
                    //Si seleccionem la opció per anar al buscador de llibres
                    startActivity(new Intent(DetallsLlibre.this,BuscadorLlibres.class));
                }
                else if(id==2){
                    //Si seleccionem la opció per escanejar un codi QR

                    //Comprovem que tinguem permisos per accedir a la camera, si no els tenim els demanem
                    if (ContextCompat.checkSelfPermission(DetallsLlibre.this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {

                        ActivityCompat.requestPermissions(DetallsLlibre.this, new String[]{Manifest.permission.CAMERA},1);
                    }
                    //Si ja tenim permisos anem directament al lector QR
                    else{
                        startActivity(new Intent(DetallsLlibre.this,InstruccionsQR.class));
                    }
                }
                else if(id==3){
                    //Si seleccionem la opció per mostrar els llibres en prestec de l'usuari
                    startActivity(new Intent(DetallsLlibre.this,LlistaReserves.class));
                }
            }
        });

        drawerToggle = new ActionBarDrawerToggle(this, drawer_menu_navegacio, R.string.menu_navegacio_obert, R.string.menu_navegacio_tancat) {

            /** Quan el menu esta obert per complert canviem el titol de la activitat */
            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                getSupportActionBar().setTitle("Navegador");
                invalidateOptionsMenu();
            }

            /** Quan el menu esta tancat per complert canviem el titol de la activitat */
            public void onDrawerClosed(View view) {
                super.onDrawerClosed(view);
                getSupportActionBar().setTitle("Detalls del llibre");
                invalidateOptionsMenu();
            }
        };

        drawerToggle.setDrawerIndicatorEnabled(true);
        drawer_menu_navegacio.addDrawerListener(drawerToggle);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeButtonEnabled(true);
    }

    //Per a evitar que el menu no estigui sincronitzat cridrem syncState
    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        drawerToggle.syncState();
    }

    //Per a evitar que el menu no estigui sincronitzat quan per exemple passem de orientacio vertical a horitzontal cridem onConfigurationChanged
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        drawerToggle.onConfigurationChanged(newConfig);
    }

    //Metodes que crea el menu de la part superior dreta, on es pot accedir a fer el login/logout
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_usuari, menu);

        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int itemSeleccionat = item.getItemId();

        if(itemSeleccionat == R.id.menu_detalls_usuari){

            SharedPreferences sp= getSharedPreferences("Login",0);
            String idUsuari  = sp.getString("idUsuari",null);

            if(idUsuari == null || idUsuari.equals("-1")){
                startActivity(new Intent(DetallsLlibre.this,Login.class));
            }
            else{
                startActivity(new Intent(DetallsLlibre.this,InfoUsuari.class));
            }
            return true;
        }
        if (drawerToggle.onOptionsItemSelected(item)) {
            return true;
        }
        return super.onOptionsItemSelected(item);

    }


    //Metode que s'executa quan l'usuari respon a la otorgació o denegació de permisos
    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case 1: {
                //Si l'usuari ha donat permisos obrim el lector QR
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

                    startActivity(new Intent(DetallsLlibre.this,InstruccionsQR.class));

                } else {
                    //En cas contrari mostrem un missatge informant que la opció no esta habilitada
                    Toast.makeText(getApplicationContext(), "S'ha denegat el permis per accedir a la camera", Toast.LENGTH_LONG).show();
                }
            }
        }
    }

}
