package com.tfg.andreu.biblioteca;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.os.Bundle;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.util.SparseArray;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.vision.CameraSource;
import com.google.android.gms.vision.Detector;
import com.google.android.gms.vision.barcode.Barcode;
import com.google.android.gms.vision.barcode.BarcodeDetector;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import org.json.JSONObject;

import java.io.IOException;
import java.util.ArrayList;

import cz.msebera.android.httpclient.Header;

public class EscanerQR extends AppCompatActivity {
    private BarcodeDetector detectorQR;
    private CameraSource camara;
    private SurfaceView sv_camara;
    private TextView valor_qr;
    private ProgressDialog dialegProgres;
    private Boolean llegirNomesUn; //Variable que servira per controlar que unicament llegim un codi QR cada cop que obrim el lector
    private int toastRecount; //Variable que s'utilitza per a no mostrar massa Toast en cas que es llegeixi un codi QR erroni

    //Menu de navegació
    private ListView lv_llista_menu;
    private DrawerLayout drawer_menu_navegacio;
    private ActionBarDrawerToggle drawerToggle;

    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.activity_escaner_qr);
        getSupportActionBar().setTitle("Escaner QR"); //Indiquem que estem a la activitat per a escanejar un codi QR

        llegirNomesUn = true;
        toastRecount = 5;

        /*En cas que el servidor no sigui instantani retornant les dades es mostra un missatge a l'usuari indicant que s'esta treballant en la seva petició*/
        dialegProgres = new ProgressDialog(EscanerQR.this);
        dialegProgres.setMessage("Obtenint informació...");
        dialegProgres.setCancelable(false);

        sv_camara = (SurfaceView) findViewById(R.id.sv_camara);
        valor_qr = (TextView) findViewById(R.id.tv_valor_qr);

        detectorQR = new BarcodeDetector.Builder(this).setBarcodeFormats(Barcode.QR_CODE).build();

        camara = new CameraSource.Builder(this, detectorQR).setRequestedPreviewSize(1600, 1024).setAutoFocusEnabled(true).build();

        sv_camara.getHolder().addCallback(new SurfaceHolder.Callback() {
            @Override
            public void surfaceCreated(SurfaceHolder holder) {
                try {
                    //noinspection MissingPermission
                    camara.start(sv_camara.getHolder());
                } catch (IOException e) {

                    e.printStackTrace();
                }
            }

            @Override
            public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {}

            @Override
            public void surfaceDestroyed(SurfaceHolder holder) {
                camara.stop();
            }
        });

        detectorQR.setProcessor(new Detector.Processor<Barcode>() {
            @Override
            public void release() {
            }

            @Override
            public void receiveDetections(Detector.Detections<Barcode> codisDetectats) {

                final SparseArray<Barcode> llistaCodis = codisDetectats.getDetectedItems();
                if (llistaCodis.size() != 0 && llegirNomesUn==true) {
                    //En el moment que llegim el codi QR busquem la informació del llibre

                    llegirNomesUn=false;
                    valor_qr.post(new Runnable() {
                        @Override
                        public void run() {
                            //Obtenim el id del llibre a traves del codi QR escanejat
                            String valorQR = llistaCodis.valueAt(0).displayValue;
                            try {

                                Integer.parseInt(valorQR);//Comprovem que el valor escanejat sigui un integer
                                valor_qr.setText(R.string.qr_detectat);

                                RequestParams params = new RequestParams();
                                params.put("idLlibre", valorQR);
                                invocarServidorDetallsLlibreQR(params);

                            }catch(NumberFormatException e){
                                //If per evitar mostrar massa toast si es llegeixen multiples codis erronis en poc temps
                                if (toastRecount >= 5) {
                                    Toast.makeText(getApplicationContext(), "Codi QR erroni", Toast.LENGTH_LONG).show();
                                    toastRecount=0;
                                }
                                else{
                                    toastRecount++;
                                }
                                valor_qr.setText(R.string.no_qr_detectat);
                                llegirNomesUn=true;//Es possible que el escaneig falli, per tant habilitem poder tornar a escanejar un cop mes
                            }

                        }
                    });
                }
            }
        });


    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        camara.release();
        detectorQR.release();
    }

    private void invocarServidorDetallsLlibreQR(RequestParams params){

        dialegProgres.show();

        AsyncHttpClient client = new AsyncHttpClient();
        client.get(getString(R.string.domini_servidor)+"/biblioteca/cerca/fescercasimple",params ,new AsyncHttpResponseHandler() {

            /*S'executa el metode onSuccess si la resposta rebuda conte el codi de resposta Http 200*/
            @Override
            public void onSuccess(int statusCode, Header[] headers, byte[] response) {

                dialegProgres.hide();
                try {
                    //Si el servidor retorna una resposta buida salta la exception i vol dir que el codi llegit no correspon a cap llibre
                    JSONObject llibreJSON = new JSONObject(new String(response, "UTF-8"));

                    Intent nouIntent = new Intent(EscanerQR.this, DetallsLlibre.class);
                    nouIntent.putExtra("Llibre", llibreJSON.toString());
                    startActivity(nouIntent);
                    finish();


                }catch (Exception e){
                    //If per evitar mostrar massa toast si es llegeixen multiples codis erronis en poc temps
                    if (toastRecount >= 5) {
                        Toast.makeText(getApplicationContext(), "Codi QR erroni", Toast.LENGTH_LONG).show();
                        toastRecount=0;
                    }
                    else{
                        toastRecount++;
                    }
                    llegirNomesUn=true; //Es possible que el escaneig falli, per tant habilitem poder tornar a escanejar un cop mes
                }

            }

            /*Si la resposta del servidor NO conte el codi Http de resposta 200 s'executa el metode onFailure*/
            @Override
            public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {

                dialegProgres.hide();
                llegirNomesUn=true;//Es possible que el escaneig falli, per tant habilitem poder tornar a escanejar un cop mes
                //Depenent del codi de resposta d'error indiquem un missatge o un altre

                if(statusCode == 500){
                    Toast.makeText(getApplicationContext(), "500. Error del servidor", Toast.LENGTH_LONG).show();
                }
                else if(statusCode == 404){
                    Toast.makeText(getApplicationContext(), "404. Recurs no trobat", Toast.LENGTH_LONG).show();
                }
                else{
                    Toast.makeText(getApplicationContext(), "Dispositiu sense connexió o servidor no disponible", Toast.LENGTH_LONG).show();
                }
            }
        });
}
    ///////////////////////////////////////
    //CONFIGURACIÓ DEL MENU DE NAVEGACIÓ///
    ///////////////////////////////////////

    //Executem la creació del menu onStart per si un usuari logeja al telefon actualitzar el menu i mostrar la opció d'anar al llistat de llibres en prestec
    @Override
    protected void onStart() {
        super.onStart();
        afegirMenuNavegacio(); //Metode que configura el menu de navegació
    }

    private void afegirMenuNavegacio(){

        lv_llista_menu = (ListView)findViewById(R.id.lv_llista_menu);
        drawer_menu_navegacio = (DrawerLayout)findViewById(R.id.drawer_menu_navegacio);

        //Afegim les opcions al menu i el click listener

        SharedPreferences sp= getSharedPreferences("Login",0);
        String idUsuari  = sp.getString("idUsuari",null);

        ArrayList<String> llistaOpcionsMenu = new ArrayList<String>();

        if(idUsuari == null || idUsuari.equals("-1")){
            llistaOpcionsMenu.add("Inici");
            llistaOpcionsMenu.add("Buscador de llibres");
        }
        else{
            llistaOpcionsMenu.add("Inici");
            llistaOpcionsMenu.add("Buscador de llibres");
            llistaOpcionsMenu.add("Llibres en prestec");
        }

        lv_llista_menu.setAdapter(new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, llistaOpcionsMenu));

        lv_llista_menu.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(id==0){
                    //Si seleccionem la opció per anar a l'inici
                    startActivity(new Intent(EscanerQR.this,Home.class));
                }

                else if(id==1){
                    //Si seleccionem la opció per anar al buscador de llibres
                    startActivity(new Intent(EscanerQR.this,BuscadorLlibres.class));
                }
                else if(id==2){
                    //Si seleccionem la opció per mostrar els llibres en prestec de l'usuari
                    startActivity(new Intent(EscanerQR.this,LlistaReserves.class));
                }
            }
        });

        drawerToggle = new ActionBarDrawerToggle(this, drawer_menu_navegacio, R.string.menu_navegacio_obert, R.string.menu_navegacio_tancat) {

            /** Quan el menu esta obert per complert canviem el titol de la activitat */
            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                getSupportActionBar().setTitle("Navegador");
                invalidateOptionsMenu();
            }

            /** Quan el menu esta tancat per complert canviem el titol de la activitat */
            public void onDrawerClosed(View view) {
                super.onDrawerClosed(view);
                getSupportActionBar().setTitle("Escaner QR");
                invalidateOptionsMenu();
            }
        };

        drawerToggle.setDrawerIndicatorEnabled(true);
        drawer_menu_navegacio.addDrawerListener(drawerToggle);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeButtonEnabled(true);
    }

    //Per a evitar que el menu no estigui sincronitzat cridrem syncState
    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        drawerToggle.syncState();
    }

    //Per a evitar que el menu no estigui sincronitzat quan per exemple passem de orientacio vertical a horitzontal cridem onConfigurationChanged
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        drawerToggle.onConfigurationChanged(newConfig);
    }

    //Metodes que crea el menu de la part superior dreta, on es pot accedir a fer el login/logout
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_usuari, menu);

        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int itemSeleccionat = item.getItemId();

        if(itemSeleccionat == R.id.menu_detalls_usuari){

            SharedPreferences sp= getSharedPreferences("Login",0);
            String idUsuari  = sp.getString("idUsuari",null);

            if(idUsuari == null || idUsuari.equals("-1")){
                startActivity(new Intent(EscanerQR.this,Login.class));
            }
            else{
                startActivity(new Intent(EscanerQR.this,InfoUsuari.class));
            }
            return true;
        }
        if (drawerToggle.onOptionsItemSelected(item)) {
            return true;
        }
        return super.onOptionsItemSelected(item);

    }
}
