package com.tfg.andreu.biblioteca;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons

import android.Manifest;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import org.json.JSONArray;

import java.util.ArrayList;

import cz.msebera.android.httpclient.Header;

public class Home extends AppCompatActivity {

    private Button bt_busca_llibres;
    private Button bt_escaner_qr;
    private Button bt_llibres_en_prestec;
    private TextView tv_home_recordatori_tornada;
    private TextView tv_home_recordatori_recollida;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_home);
        getSupportActionBar().setTitle("Inici");//Indiquem que estem a la activitat inicial de l'aplicació


        bt_busca_llibres = (Button) findViewById(R.id.bt_busca_llibres);
        bt_escaner_qr = (Button) findViewById(R.id.bt_escaner_qr);
        bt_llibres_en_prestec = (Button) findViewById(R.id.bt_llibres_en_prestec);
        tv_home_recordatori_tornada = (TextView) findViewById(R.id.tv_home_recordatori_tornada);
        tv_home_recordatori_recollida = (TextView) findViewById(R.id.tv_home_recordatori_recollida);
        tv_home_recordatori_tornada.setText("\nObtenint informació dels llibres que has de tornar...\n\n");
        tv_home_recordatori_recollida.setText("\nObtenint informació dels llibres que pots recollir...\n\n");

        /*Afegim el listener al boto per buscar llibres i per mostrar els llibres que es tenen en prestec*/
        bt_busca_llibres.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                startActivity(new Intent(Home.this,BuscadorLlibres.class));
            }
        });



        bt_llibres_en_prestec.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                startActivity(new Intent(Home.this,LlistaReserves.class));
            }
        });

        bt_escaner_qr.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                //Comprovem que tinguem permisos per accedir a la camera, si no els tenim els demanem
                if (ContextCompat.checkSelfPermission(Home.this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {

                        ActivityCompat.requestPermissions(Home.this, new String[]{Manifest.permission.CAMERA},1);
                }
                //Si ja tenim permisos anem a les instruccions per a utilitzar el lector lector QR
                else{
                    startActivity(new Intent(Home.this,InstruccionsQR.class));
                }
            }
        });
    }

    //Executem onStart per mostrar / ocultar el boto per accedir a la llista de llibres en prestec depenent de si hi ha un usuari logejat al telefon o no.
    @Override
    protected void onStart() {
        super.onStart();

        SharedPreferences sp= getSharedPreferences("Login",0);
        String idUsuari  = sp.getString("idUsuari",null);

        if(idUsuari == null || idUsuari.equals("-1")){
            bt_llibres_en_prestec.setVisibility(View.INVISIBLE);
            tv_home_recordatori_tornada.setText("");
            tv_home_recordatori_tornada.setVisibility(View.INVISIBLE);
            tv_home_recordatori_recollida.setText("");
            tv_home_recordatori_recollida.setVisibility(View.INVISIBLE);
        }
        else{
            //Obtenim la informació dels recordatoris per a tornar llibres
            bt_llibres_en_prestec.setVisibility(View.VISIBLE);
            tv_home_recordatori_tornada.setVisibility(View.VISIBLE);
            RequestParams params = new RequestParams();
            params.put("idUsuari", idUsuari);
            invocarServidorRecordatoriRetorn(params);

            //Obtenim la informació del recordatori per a recollir llibres
            tv_home_recordatori_recollida.setVisibility(View.VISIBLE);
            invocarServidorRecordatoriRecollida(params);

        }

    }

    //Metode que s'executa quan l'usuari respon a la otorgació o denegació de permisos
    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case 1: {
                //Si l'usuari ha donat permisos obrim les instruccions del lector QR
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

                    startActivity(new Intent(Home.this,InstruccionsQR.class));

                } else {
                    //En cas contrari deshabilitem el boto
                    bt_escaner_qr.setEnabled(false);
                }
            }
        }
    }

   private void invocarServidorRecordatoriRetorn(RequestParams params){

        AsyncHttpClient client = new AsyncHttpClient();
        client.get(getString(R.string.domini_servidor)+"/biblioteca/infoRetorn/info",params ,new AsyncHttpResponseHandler() {

            /*S'executa el metode onSuccess si la resposta rebuda conte el codi de resposta Http 200*/
            @Override
            public void onSuccess(int statusCode, Header[] headers, byte[] response) {

                ArrayList<Recordatori> llistaRecordatoris = new ArrayList<Recordatori>();/*Llista que contindra tots els objectes Recordatori rebuts del servidor*/
                int i;
                try {
                    //Transformem la resposta del servidor de un array de bites a un string i a continuació a un array JSON
                    JSONArray llistaRecordatorisJson = new JSONArray(new String(response, "UTF-8"));

                    //Si hi ha recordatoris que mostrar ->
                    if(llistaRecordatorisJson.length()>0) {

                        /*Transformem l'array JSON a un array d'objectes Llibre*/
                        for (i = 0; i < llistaRecordatorisJson.length(); i++) {
                            llistaRecordatoris.add(new Recordatori(llistaRecordatorisJson.getJSONObject(i)));
                        }
                        tv_home_recordatori_tornada.setText("\nRecorda! Has de retornar els seguents llibres:\n\n");

                        for (i = 0; i < llistaRecordatoris.size(); i++) {
                            tv_home_recordatori_tornada.append("-"+llistaRecordatoris.get(i).getTitolLlibre()+" el dia "+llistaRecordatoris.get(i).getData()+"\n\n");
                        }

                    }
                    else
                    {
                        tv_home_recordatori_tornada.setText("\nNo has de retornar cap llibre proximament\n\n");
                    }
                }catch (Exception e){
                    Log.d("Error","Error en la resposta del servidor (El JSON rebut del servidor pot ser invalid?");
                    e.printStackTrace();
                }

            }

            /*Si la resposta del servidor NO conte el codi Http de resposta 200 s'executa el metode onFailure*/
            @Override
            public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {
                //Depenent del codi de resposta d'error indiquem un missatge o un altre
                tv_home_recordatori_tornada.setText("\nNo es pot accedir a la informació dels llibres que s'han de tornar\n\n");
                if(statusCode == 500){
                    Toast.makeText(getApplicationContext(), "500. Error del servidor", Toast.LENGTH_LONG).show();

                }
                else if(statusCode == 404){
                    Toast.makeText(getApplicationContext(), "404. Recurs no trobat", Toast.LENGTH_LONG).show();

                }
                else{
                    Toast.makeText(getApplicationContext(), "Dispositiu sense connexió o servidor no disponible", Toast.LENGTH_LONG).show();

                }
            }
        });

    }

    private void invocarServidorRecordatoriRecollida(RequestParams params){

        AsyncHttpClient client = new AsyncHttpClient();
        client.get(getString(R.string.domini_servidor)+"/biblioteca/infoRecollida/info",params ,new AsyncHttpResponseHandler() {

            /*S'executa el metode onSuccess si la resposta rebuda conte el codi de resposta Http 200*/
            @Override
            public void onSuccess(int statusCode, Header[] headers, byte[] response) {

                ArrayList<Recordatori> llistaRecordatoris = new ArrayList<Recordatori>();/*Llista que contindra tots els objectes Recordatori rebuts del servidor*/
                int i;
                try {
                    //Transformem la resposta del servidor de un array de bites a un string i a continuació a un array JSON
                    JSONArray llistaRecordatorisJson = new JSONArray(new String(response, "UTF-8"));

                    //Si hi ha recordatoris que mostrar ->
                    if(llistaRecordatorisJson.length()>0) {

                        /*Transformem l'array JSON a un array d'objectes Llibre*/
                        for (i = 0; i < llistaRecordatorisJson.length(); i++) {
                            llistaRecordatoris.add(new Recordatori(llistaRecordatorisJson.getJSONObject(i)));
                        }
                        tv_home_recordatori_recollida.setText("\nRecorda! Pots recollir els seguents llibres:\n\n");

                        for (i = 0; i < llistaRecordatoris.size(); i++) {
                            tv_home_recordatori_recollida.append("-"+llistaRecordatoris.get(i).getTitolLlibre()+" a partir del dia "+llistaRecordatoris.get(i).getData()+"\n\n");
                        }

                    }
                    else
                    {
                        tv_home_recordatori_recollida.setText("\nNo hi ha cap llibre per a recollir\n\n");
                    }
                }catch (Exception e){
                    Log.d("Error","Error en la resposta del servidor (El JSON rebut del servidor pot ser invalid?");
                    e.printStackTrace();
                }

            }

            /*Si la resposta del servidor NO conte el codi Http de resposta 200 s'executa el metode onFailure*/
            @Override
            public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {
                //Depenent del codi de resposta d'error indiquem un missatge o un altre
                tv_home_recordatori_recollida.setText("\nNo es pot accedir a la informació dels recordatoris\n\n");
                if(statusCode == 500){
                    Toast.makeText(getApplicationContext(), "500. Error del servidor", Toast.LENGTH_LONG).show();

                }
                else if(statusCode == 404){
                    Toast.makeText(getApplicationContext(), "404. Recurs no trobat", Toast.LENGTH_LONG).show();

                }
                else{
                    Toast.makeText(getApplicationContext(), "Dispositiu sense connexió o servidor no disponible", Toast.LENGTH_LONG).show();

                }
            }
        });

    }

   //Metodes que crea el menu de la part superior dreta, on es pot accedir a fer el login/logout
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_usuari, menu);

        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int itemSeleccionat = item.getItemId();

        if(itemSeleccionat == R.id.menu_detalls_usuari){

            SharedPreferences sp= getSharedPreferences("Login",0);
            String idUsuari  = sp.getString("idUsuari",null);

            if(idUsuari == null || idUsuari.equals("-1")){
                startActivity(new Intent(Home.this,Login.class));
            }
            else{
                startActivity(new Intent(Home.this,InfoUsuari.class));
            }
            return true;
        }

        return super.onOptionsItemSelected(item);

    }

}
