package com.tfg.andreu.biblioteca;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons
import android.Manifest;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;

//Activitat que serveix per donar la funcionalitat de logout, juntament mostrant la informació del usuari que esta logejat al sistema actualment.
public class InfoUsuari extends AppCompatActivity {

    private TextView nom_usuari;
    private Button bt_logout;

    //Menu de navegació
    private ListView lv_llista_menu;
    private DrawerLayout drawer_menu_navegacio;
    private ActionBarDrawerToggle drawerToggle;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_info_usuari);
        getSupportActionBar().setTitle("Usuari");//Indiquem que estem a la activitat per a mostrar el usuari logejat
        nom_usuari = (TextView) findViewById(R.id.tv_nom_usuari);

        SharedPreferences sp= getSharedPreferences("Login",0);
        nom_usuari.setText(sp.getString("nomUsuari",null));

        bt_logout = (Button) findViewById(R.id.bt_tanca_sessio);

        bt_logout.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {

                //Eliminem l'usuari de les sharedPreferences, per tant fem logout amb ell.
                SharedPreferences preferences = getSharedPreferences("Login",0);
                SharedPreferences.Editor editor = preferences.edit();
                editor.clear();
                editor.commit();

                Toast.makeText(getApplicationContext(), "Sessió tancada correctament", Toast.LENGTH_LONG).show();
                finish();
            }
        });
        //En aquest cas no l'activo onStart() ja que en aquesta activitat no es pot fer logout i seguir sense sortir
        afegirMenuNavegacio(); //Metode que configura el menu de navegació
    }

    ///////////////////////////////////////
    //CONFIGURACIÓ DEL MENU DE NAVEGACIÓ///
    ///////////////////////////////////////

    private void afegirMenuNavegacio(){

        lv_llista_menu = (ListView)findViewById(R.id.lv_llista_menu);
        drawer_menu_navegacio = (DrawerLayout)findViewById(R.id.drawer_menu_navegacio);

        //Afegim les opcions al menu i el click listener

        ArrayList<String> llistaOpcionsMenu = new ArrayList<String>();

            llistaOpcionsMenu.add("Inici");
            llistaOpcionsMenu.add("Buscador de llibres");
            llistaOpcionsMenu.add("Escanejar codi QR");
            llistaOpcionsMenu.add("Llibres en prestec");


        lv_llista_menu.setAdapter(new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, llistaOpcionsMenu));

        lv_llista_menu.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(id==0){
                    //Si seleccionem la opció per anar a l'inici
                    startActivity(new Intent(InfoUsuari.this,Home.class));
                }
                else if(id==1){
                    //Si seleccionem la opció per anar al buscador de llibres
                    startActivity(new Intent(InfoUsuari.this,BuscadorLlibres.class));
                }
                else if(id==2){
                    //Si seleccionem la opció per escanejar un codi QR

                    //Comprovem que tinguem permisos per accedir a la camera, si no els tenim els demanem
                    if (ContextCompat.checkSelfPermission(InfoUsuari.this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {

                        ActivityCompat.requestPermissions(InfoUsuari.this, new String[]{Manifest.permission.CAMERA},1);
                    }
                    //Si ja tenim permisos anem directament al lector QR
                    else{
                        startActivity(new Intent(InfoUsuari.this,InstruccionsQR.class));
                    }
                }
                else if(id==3){
                    //Si seleccionem la opció per mostrar els llibres en prestec de l'usuari
                    startActivity(new Intent(InfoUsuari.this,LlistaReserves.class));
                }
            }
        });

        drawerToggle = new ActionBarDrawerToggle(this, drawer_menu_navegacio, R.string.menu_navegacio_obert, R.string.menu_navegacio_tancat) {

            /** Quan el menu esta obert per complert canviem el titol de la activitat */
            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                getSupportActionBar().setTitle("Navegador");
                invalidateOptionsMenu();
            }

            /** Quan el menu esta tancat per complert canviem el titol de la activitat */
            public void onDrawerClosed(View view) {
                super.onDrawerClosed(view);
                getSupportActionBar().setTitle("Usuari");
                invalidateOptionsMenu();
            }
        };

        drawerToggle.setDrawerIndicatorEnabled(true);
        drawer_menu_navegacio.addDrawerListener(drawerToggle);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeButtonEnabled(true);
    }

    //Per a evitar que el menu no estigui sincronitzat cridrem syncState
    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        drawerToggle.syncState();
    }

    //Per a evitar que el menu no estigui sincronitzat quan per exemple passem de orientacio vertical a horitzontal cridem onConfigurationChanged
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        drawerToggle.onConfigurationChanged(newConfig);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        if (drawerToggle.onOptionsItemSelected(item)) {
            return true;
        }
        return super.onOptionsItemSelected(item);

    }


    //Metode que s'executa quan l'usuari respon a la otorgació o denegació de permisos
    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case 1: {
                //Si l'usuari ha donat permisos obrim el lector QR
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

                    startActivity(new Intent(InfoUsuari.this,InstruccionsQR.class));

                } else {
                    //En cas contrari mostrem un missatge informant que la opció no esta habilitada
                    Toast.makeText(getApplicationContext(), "S'ha denegat el permis per accedir a la camera", Toast.LENGTH_LONG).show();
                }
            }
        }
    }

}



