package com.tfg.andreu.biblioteca;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons
import android.Manifest;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.Toast;
import com.loopj.android.http.*;
import org.json.JSONArray;
import java.util.ArrayList;
import cz.msebera.android.httpclient.Header;

//Activitat que mostra tot el llistat de llibres reservats que te l'usuari
public class LlistaReserves extends AppCompatActivity {

    private RecyclerView rvLlistaReserves;
    private AdaptadorLlistaReserves adaptadorLlistaReserves;
    private ProgressDialog dialegProgres;

    private RelativeLayout rl_reserves_llista_buida;

    //Menu de navegació
    private ListView lv_llista_menu;
    private DrawerLayout drawer_menu_navegacio;
    private ActionBarDrawerToggle drawerToggle;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_llista_reserves);
        getSupportActionBar().setTitle("Llibres en préstec");//Indiquem que estem a la activitat per a mostrar el llistat de reserves

        rvLlistaReserves = (RecyclerView) findViewById(R.id.rv_llista_reserves);
        rl_reserves_llista_buida = (RelativeLayout) findViewById(R.id.rl_reserves_llista_buida);
        rl_reserves_llista_buida.setVisibility(View.GONE);

       dialegProgres = new ProgressDialog(LlistaReserves.this);
       dialegProgres.setMessage("Carregant llista de reserves");
       dialegProgres.setCancelable(false);

        //En aquest cas no cal executar dintre de onStart() ja que la opció per entrar a la llista de reserves no te sentit aqui
        afegirMenuNavegacio(); //Metode que configura el menu de navegació
    }

    /*La petició per obtenir la informació de les reserves de llibres s'executa onStart() per tal de que s'actualitzi si s'amplia el temsp de prestec d'un llibre
    */
    @Override
    protected void onStart() {
        super.onStart();

        SharedPreferences sp = getSharedPreferences("Login", 0);
        String idUsuari = sp.getString("idUsuari", null);

        if(idUsuari == null || idUsuari.equals("-1")){

            startActivity(new Intent(LlistaReserves.this,Login.class));
        }
        else {

                RequestParams params = new RequestParams();
                params.put("idUsuari", idUsuari);
                invocarServidorReserves(params);
        }
    }

    /**
     * Aquest metode executa una petició GET al servidor de l'aplicació amb els parametres indicats
     * Tambe s'encarrega de vincular les diferents parts del RecyclerView en cas que la petició tingui exit.
     * S'utilitza la llibreria android-async-http
     */
    private void invocarServidorReserves(RequestParams params){

        /*En cas que el servidor no sigui instantani retornant les dades es mostra un missatge a l'usuari indicant que s'esta treballant en la seva petició*/
        dialegProgres.show();

        AsyncHttpClient client = new AsyncHttpClient();
        client.get(getString(R.string.domini_servidor)+"/biblioteca/reservesUsuari/obtenirLlista",params ,new AsyncHttpResponseHandler() {

            /*S'executa el metode onSuccess si la resposta rebuda conte el codi de resposta Http 200*/
            @Override
            public void onSuccess(int statusCode, Header[] headers, byte[] response) {

                dialegProgres.hide();
                dialegProgres.dismiss();

                ArrayList<Reserva> llistaInfoReserves = new ArrayList<Reserva>();/*Llista que contindra tots els objectes Reserva rebuts del servidor*/

                try {
                    //Transformem la resposta del servidor de un array de bites a un string i a continuació a un array JSON
                    JSONArray infoReservesJson = new JSONArray(new String(response, "UTF-8"));

                    if (infoReservesJson.length()==0){
                        rvLlistaReserves.setVisibility(View.GONE);
                        rl_reserves_llista_buida.setVisibility(View.VISIBLE);
                    }
                    else {
                        rl_reserves_llista_buida.setVisibility(View.GONE);
                        rvLlistaReserves.setVisibility(View.VISIBLE);

                        /*Transformem l'array JSON a un array d'objectes Reserva*/
                        for (int i = 0; i < infoReservesJson.length(); i++) {
                            llistaInfoReserves.add(new Reserva(infoReservesJson.getJSONObject(i)));
                        }

                        /*Vinculem l'adaptador amb el RecyclerView i indiquem el layoutManager que volem*/

                        adaptadorLlistaReserves = new AdaptadorLlistaReserves(llistaInfoReserves);
                        rvLlistaReserves.setAdapter(adaptadorLlistaReserves);
                        rvLlistaReserves.setLayoutManager(new LinearLayoutManager(LlistaReserves.this));

                        /*Com la grandaria del array de llibres no canviara dinamicament podem millorar el rendiment amb aquesta propietat*/
                        rvLlistaReserves.setHasFixedSize(true);
                    }
                } catch (Exception e) {
                    Log.d("Error","Error en la resposta del servidor (El JSON rebut del servidor pot ser invalid?");
                    e.printStackTrace();
                }
            }

            /*Si la resposta del servidor NO conte el codi Http de resposta 200 s'executa el metode onFailure*/
            @Override
            public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {

                dialegProgres.hide();
                dialegProgres.dismiss();
                //Depenent del codi de resposta d'error indiquem un missatge o un altre

                if(statusCode == 500){
                    Toast.makeText(getApplicationContext(), "500. Error del servidor", Toast.LENGTH_LONG).show();
                }
                else if(statusCode == 404){
                    Toast.makeText(getApplicationContext(), "404. Recurs no trobat", Toast.LENGTH_LONG).show();
                }
                else{
                    Toast.makeText(getApplicationContext(), "Dispositiu sense connexió o servidor no disponible", Toast.LENGTH_LONG).show();
                }
            }
        });

    }


    ///////////////////////////////////////
    //CONFIGURACIÓ DEL MENU DE NAVEGACIÓ///
    ///////////////////////////////////////

    private void afegirMenuNavegacio(){

        lv_llista_menu = (ListView)findViewById(R.id.lv_llista_menu);
        drawer_menu_navegacio = (DrawerLayout)findViewById(R.id.drawer_menu_navegacio);

        //Afegim les opcions al menu i el click listener

        SharedPreferences sp= getSharedPreferences("Login",0);
        String idUsuari  = sp.getString("idUsuari",null);

        ArrayList<String> llistaOpcionsMenu = new ArrayList<String>();


            llistaOpcionsMenu.add("Inici");
            llistaOpcionsMenu.add("Buscador de llibres");
            llistaOpcionsMenu.add("Escanejar codi QR");
        lv_llista_menu.setAdapter(new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, llistaOpcionsMenu));

        lv_llista_menu.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(id==0){
                    //Si seleccionem la opció per anar a l'inici
                    startActivity(new Intent(LlistaReserves.this,Home.class));
                }

                else if(id==1){
                    //Si seleccionem la opció per anar al buscador de llibres
                    startActivity(new Intent(LlistaReserves.this,BuscadorLlibres.class));
                }
                else if(id==2){
                    //Si seleccionem la opció per escanejar un codi QR

                    //Comprovem que tinguem permisos per accedir a la camera, si no els tenim els demanem
                    if (ContextCompat.checkSelfPermission(LlistaReserves.this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {

                        ActivityCompat.requestPermissions(LlistaReserves.this, new String[]{Manifest.permission.CAMERA},1);
                    }
                    //Si ja tenim permisos anem directament al lector QR
                    else{
                        startActivity(new Intent(LlistaReserves.this,InstruccionsQR.class));
                    }
                }

            }
        });

        drawerToggle = new ActionBarDrawerToggle(this, drawer_menu_navegacio, R.string.menu_navegacio_obert, R.string.menu_navegacio_tancat) {

            /** Quan el menu esta obert per complert canviem el titol de la activitat */
            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                getSupportActionBar().setTitle("Navegador");
                invalidateOptionsMenu();
            }

            /** Quan el menu esta tancat per complert canviem el titol de la activitat */
            public void onDrawerClosed(View view) {
                super.onDrawerClosed(view);
                getSupportActionBar().setTitle("Llibres en préstec");
                invalidateOptionsMenu();
            }
        };

        drawerToggle.setDrawerIndicatorEnabled(true);
        drawer_menu_navegacio.addDrawerListener(drawerToggle);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeButtonEnabled(true);
    }

    //Per a evitar que el menu no estigui sincronitzat cridrem syncState
    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        drawerToggle.syncState();
    }

    //Per a evitar que el menu no estigui sincronitzat quan per exemple passem de orientacio vertical a horitzontal cridem onConfigurationChanged
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        drawerToggle.onConfigurationChanged(newConfig);

    }

    //Metodes que crea el menu de la part superior dreta, on es pot accedir a fer el login/logout
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_usuari, menu);

        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int itemSeleccionat = item.getItemId();

        if(itemSeleccionat == R.id.menu_detalls_usuari){

            SharedPreferences sp= getSharedPreferences("Login",0);
            String idUsuari  = sp.getString("idUsuari",null);

            if(idUsuari == null || idUsuari.equals("-1")){
                startActivity(new Intent(LlistaReserves.this,Login.class));
            }
            else{
                startActivity(new Intent(LlistaReserves.this,InfoUsuari.class));
            }
            return true;
        }
        if (drawerToggle.onOptionsItemSelected(item)) {
            return true;
        }
        return super.onOptionsItemSelected(item);

    }


    //Metode que s'executa quan l'usuari respon a la otorgació o denegació de permisos
    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case 1: {
                //Si l'usuari ha donat permisos obrim el lector QR
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

                    startActivity(new Intent(LlistaReserves.this,InstruccionsQR.class));

                } else {
                    //En cas contrari mostrem un missatge informant que la opció no esta habilitada
                    Toast.makeText(getApplicationContext(), "S'ha denegat el permis per accedir a la camera", Toast.LENGTH_LONG).show();
                }
            }
        }
    }

}
