package com.tfg.andreu.biblioteca;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons
import android.Manifest;
import android.app.ProgressDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import org.json.JSONObject;

import java.util.ArrayList;

import cz.msebera.android.httpclient.Header;

//Activitat que s'utilitza per a realitzar el login a l'aplicació
public class Login extends AppCompatActivity {

    private EditText usuari;
    private EditText password;
    private Button bt_login;
    private ProgressDialog dialegProgres;

    //Menu de navegació
    private ListView lv_llista_menu;
    private DrawerLayout drawer_menu_navegacio;
    private ActionBarDrawerToggle drawerToggle;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);
        getSupportActionBar().setTitle("Iniciar sessió");//Indiquem que estem a la activitat per a fer login al telefon

        usuari = (EditText) findViewById(R.id.ed_usuari);
        password = (EditText) findViewById(R.id.ed_password);
        bt_login = (Button) findViewById(R.id.bt_fes_login);

        //Definim el comportament del boto de login
        bt_login.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                /*En cas que el servidor no sigui instantani retornant les dades es mostra un missatge a l'usuari indicant que s'esta treballant en la seva petició*/
                dialegProgres = new ProgressDialog(Login.this);
                dialegProgres.setMessage("Autentificant usuari...");
                dialegProgres.setCancelable(false);

                /*Realitzem la petició de login al servidor amb els parametres indicats*/
                RequestParams params = new RequestParams();
                params.put("usuari", usuari.getText());
                params.put("password", password.getText());
                invocarServidorLogin(params);
            }
        });
        //No executem a onStart() perque un cop fet el login sempre es surt de la activitat per tant no te sentit mirar si s'ha de mostrar la opció de llista de reserves
        afegirMenuNavegacio(); //Metode que configura el menu de navegació
    }

    /**
     * Aquest metode executa una petició GET al servidor de l'aplicació amb els parametres indicats
     * S'utilitza la llibreria android-async-http
     */
    private void invocarServidorLogin(RequestParams params){

        dialegProgres.show();

        AsyncHttpClient client = new AsyncHttpClient();
        client.get(getString(R.string.domini_servidor)+"/biblioteca/login/autentifica",params ,new AsyncHttpResponseHandler() {

            /*S'executa el metode onSuccess si la resposta rebuda conte el codi de resposta Http 200*/
            @Override
            public void onSuccess(int statusCode, Header[] headers, byte[] response) {

                dialegProgres.hide();
                Usuari usuari;

                try {
                    //Obtenim la resposta del servidor
                    JSONObject usuariJson = new JSONObject(new String(response, "UTF-8"));
                    usuari = new Usuari(usuariJson.getString("id"), usuariJson.getString("nomUsuari"));
                    //Si les dades proporcionades no corresponen a cap usuari idUsuari == -1
                    if(usuari.getIdUsuari().equals("-1")){

                        Toast.makeText(getApplicationContext(), "Error. Les dades proporcionades no corresponen a cap usuari", Toast.LENGTH_LONG).show();
                    }
                    else
                    {//En cas contrari guardem el usuari logejat al telefon
                        SharedPreferences sp = getSharedPreferences("Login", 0);
                        SharedPreferences.Editor Ed = sp.edit();
                        Ed.putString("idUsuari",usuari.getIdUsuari());
                        Ed.putString("nomUsuari",usuari.getNomUsuari());
                        Ed.commit();

                        Toast.makeText(getApplicationContext(), "Sessió oberta correctament", Toast.LENGTH_LONG).show();
                        finish();
                    }

                }catch (Exception e){
                    Log.d("Error","Error en la resposta del servidor (El JSON rebut del servidor pot ser invalid?");
                    e.printStackTrace();
                }
            }

            /*Si la resposta del servidor NO conte el codi Http de resposta 200 s'executa el metode onFailure*/
            @Override
            public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {

                dialegProgres.hide();
                //Depenent del codi de resposta d'error indiquem un missatge o un altre

                if(statusCode == 500){
                    Toast.makeText(getApplicationContext(), "500. Error del servidor", Toast.LENGTH_LONG).show();
                }
                else if(statusCode == 404){
                    Toast.makeText(getApplicationContext(), "404. Recurs no trobat", Toast.LENGTH_LONG).show();
                }
                else{
                    Toast.makeText(getApplicationContext(), "Dispositiu sense connexió o servidor no disponible", Toast.LENGTH_LONG).show();
                }
            }
        });

    }

    private void afegirMenuNavegacio(){

        lv_llista_menu = (ListView)findViewById(R.id.lv_llista_menu);
        drawer_menu_navegacio = (DrawerLayout)findViewById(R.id.drawer_menu_navegacio);

        //Afegim les opcions al menu i el click listener
        ArrayList<String> llistaOpcionsMenu = new ArrayList<String>();
        llistaOpcionsMenu.add("Inici");
        llistaOpcionsMenu.add("Buscador de llibres");
        llistaOpcionsMenu.add("Escanejar codi QR");

        lv_llista_menu.setAdapter(new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, llistaOpcionsMenu));

        lv_llista_menu.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(id==0){
                    //Si seleccionem la opció per anar a l'inici
                    startActivity(new Intent(Login.this,Home.class));
                    finish();//Tanquem la activitat perque considero que els usuaris no voldran tornar aqui si utilitzen el boto cap endarrere
                }

                else if(id==1){
                    //Si seleccionem la opció per anar al buscador de llibres
                    startActivity(new Intent(Login.this,BuscadorLlibres.class));
                    finish();//Tanquem la activitat perque considero que els usuaris no voldran tornar aqui si utilitzen el boto cap endarrere
                }
                else if(id==2){
                    //Si seleccionem la opció per escanejar un codi QR

                    //Comprovem que tinguem permisos per accedir a la camera, si no els tenim els demanem
                    if (ContextCompat.checkSelfPermission(Login.this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {

                        ActivityCompat.requestPermissions(Login.this, new String[]{Manifest.permission.CAMERA},1);
                    }
                    //Si ja tenim permisos anem directament al lector QR
                    else{
                        startActivity(new Intent(Login.this,InstruccionsQR.class));
                        finish();//Tanquem la activitat perque considero que els usuaris no voldran tornar aqui si utilitzen el boto cap endarrere
                    }
                }
            }
        });

        drawerToggle = new ActionBarDrawerToggle(this, drawer_menu_navegacio, R.string.menu_navegacio_obert, R.string.menu_navegacio_tancat) {

            /** Quan el menu esta obert per complert canviem el titol de la activitat */
            public void onDrawerOpened(View drawerView) {
                super.onDrawerOpened(drawerView);
                getSupportActionBar().setTitle("Navegador");
                invalidateOptionsMenu();
            }

            /** Quan el menu esta tancat per complert canviem el titol de la activitat */
            public void onDrawerClosed(View view) {
                super.onDrawerClosed(view);
                getSupportActionBar().setTitle("Iniciar sessió");
                invalidateOptionsMenu();
            }
        };

        drawerToggle.setDrawerIndicatorEnabled(true);
        drawer_menu_navegacio.addDrawerListener(drawerToggle);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setHomeButtonEnabled(true);
    }

    //Per a evitar que el menu no estigui sincronitzat cridrem syncState
    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        drawerToggle.syncState();
    }

    //Per a evitar que el menu no estigui sincronitzat quan per exemple passem de orientacio vertical a horitzontal cridem onConfigurationChanged
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        drawerToggle.onConfigurationChanged(newConfig);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        if (drawerToggle.onOptionsItemSelected(item)) {
            return true;
        }
        return super.onOptionsItemSelected(item);

    }


    //Metode que s'executa quan l'usuari respon a la otorgació o denegació de permisos
    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case 1: {
                //Si l'usuari ha donat permisos obrim el lector QR
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

                    startActivity(new Intent(Login.this,InstruccionsQR.class));
                    finish();//Tanquem la activitat perque considero que els usuaris no voldran tornar aqui si utilitzen el boto cap endarrere

                } else {
                    //En cas contrari mostrem un missatge informant que la opció no esta habilitada
                    Toast.makeText(getApplicationContext(), "S'ha denegat el permis per accedir a la camera", Toast.LENGTH_LONG).show();
                }
            }
        }
    }

}


