package com.tfg.andreu.bibliotecari;
//@Autor: Andreu Querol Salsench
// Llicencia: Aquesta obra està subjecta a una llicència de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons
import android.app.ProgressDialog;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.SparseArray;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.vision.CameraSource;
import com.google.android.gms.vision.Detector;
import com.google.android.gms.vision.barcode.Barcode;
import com.google.android.gms.vision.barcode.BarcodeDetector;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import org.json.JSONObject;

import java.io.IOException;

import cz.msebera.android.httpclient.Header;

public class TornarLlibreQR extends AppCompatActivity {

    private BarcodeDetector detectorQR;
    private CameraSource camara;
    private SurfaceView sv_camara;
    private TextView valor_qr;
    private ProgressDialog dialegProgres;
    private Boolean llegirNomesUn; //Variable que servira per controlar que unicament llegim un codi QR cada cop que obrim el lector
    private int toastRecount; //Variable que s'utilitza per a no mostrar massa Toast en cas que es llegeixi un codi QR erroni


    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.activity_tornar_llibre_qr);
        getSupportActionBar().setTitle("Escaner QR"); //Indiquem que estem a la activitat per a escanejar un codi QR

        llegirNomesUn = true;
        toastRecount = 5;

        /*En cas que el servidor no sigui instantani retornant les dades es mostra un missatge a l'usuari indicant que s'esta treballant en la seva petició*/
        dialegProgres = new ProgressDialog(TornarLlibreQR.this);
        dialegProgres.setMessage("Executan petició...");
        dialegProgres.setCancelable(false);

        sv_camara = (SurfaceView) findViewById(R.id.sv_camara);
        valor_qr = (TextView) findViewById(R.id.tv_valor_qr);

        detectorQR = new BarcodeDetector.Builder(this).setBarcodeFormats(Barcode.QR_CODE).build();

        camara = new CameraSource.Builder(this, detectorQR).setRequestedPreviewSize(1600, 1024).setAutoFocusEnabled(true).build();

        sv_camara.getHolder().addCallback(new SurfaceHolder.Callback() {
            @Override
            public void surfaceCreated(SurfaceHolder holder) {
                try {
                    //noinspection MissingPermission
                    camara.start(sv_camara.getHolder());
                } catch (IOException e) {

                    e.printStackTrace();
                }
            }

            @Override
            public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {}

            @Override
            public void surfaceDestroyed(SurfaceHolder holder) {
                camara.stop();
            }
        });

        detectorQR.setProcessor(new Detector.Processor<Barcode>() {
            @Override
            public void release() {
            }

            @Override
            public void receiveDetections(Detector.Detections<Barcode> codisDetectats) {

                final SparseArray<Barcode> llistaCodis = codisDetectats.getDetectedItems();
                if (llistaCodis.size() != 0 && llegirNomesUn==true) {
                    //En el moment que llegim el codi QR busquem la informació del llibre

                    llegirNomesUn=false;
                    valor_qr.post(new Runnable() {
                        @Override
                        public void run() {
                            //Obtenim el id del llibre a traves del codi QR escanejat
                            String valorQR = llistaCodis.valueAt(0).displayValue;
                            try {

                                Integer.parseInt(valorQR);//Comprovem que el valor escanejat sigui un integer
                                valor_qr.setText(R.string.qr_detectat);

                                RequestParams params = new RequestParams();
                                params.put("idLlibre", valorQR);
                                invocarServidorTornarLlibreQR(params);

                            }catch(NumberFormatException e){
                                //If per evitar mostrar massa toast si es llegeixen multiples codis erronis en poc temps
                                if (toastRecount >= 5) {
                                    Toast.makeText(getApplicationContext(), "Codi QR erroni", Toast.LENGTH_LONG).show();
                                    toastRecount=0;
                                }
                                else{
                                    toastRecount++;
                                }
                                valor_qr.setText(R.string.no_qr_detectat);
                                llegirNomesUn=true;//Es possible que el escaneig falli, per tant si falla habilitem poder tornar a escanejar un cop mes
                            }

                        }
                    });
                }
            }
        });


    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        camara.release();
        detectorQR.release();
    }

    private void invocarServidorTornarLlibreQR(RequestParams params){

        dialegProgres.show();

        AsyncHttpClient client = new AsyncHttpClient();
        client.get(getString(R.string.domini_servidor)+"/biblioteca/bibliotecariTL/tornarLlibre",params ,new AsyncHttpResponseHandler() {

            /*S'executa el metode onSuccess si la resposta rebuda conte el codi de resposta Http 200*/
            @Override
            public void onSuccess(int statusCode, Header[] headers, byte[] response) {

                dialegProgres.hide();
                try {
                    //Si el servidor retorna una resposta buida salta la exception i vol dir que el codi llegit no correspon a cap llibre
                    Boolean resultatOK = new JSONObject(new String(response, "UTF-8")).getBoolean("resultatOK");

                    if(resultatOK==true){
                        Toast.makeText(getApplicationContext(), "El llibre s'ha retornar correctament", Toast.LENGTH_LONG).show();
                    }
                    else
                    {
                        Toast.makeText(getApplicationContext(), "Error al retornar el llibre", Toast.LENGTH_LONG).show();
                    }
                    finish();

                }catch (Exception e){
                    //If per evitar mostrar massa toast si es llegeixen multiples codis erronis en poc temps
                    if (toastRecount >= 5) {
                        Toast.makeText(getApplicationContext(), "Codi QR erroni", Toast.LENGTH_LONG).show();
                        toastRecount=0;
                    }
                    else{
                        toastRecount++;
                    }
                    llegirNomesUn=true; //Es possible que el escaneig falli, per tant habilitem poder tornar a escanejar un cop mes
                }

            }

            /*Si la resposta del servidor NO conte el codi Http de resposta 200 s'executa el metode onFailure*/
            @Override
            public void onFailure(int statusCode, Header[] headers, byte[] errorResponse, Throwable e) {

                dialegProgres.hide();
                llegirNomesUn=true;//Es possible que el escaneig falli, per tant habilitem poder tornar a escanejar un cop mes
                //Depenent del codi de resposta d'error indiquem un missatge o un altre

                if(statusCode == 500){
                    Toast.makeText(getApplicationContext(), "500. Error del servidor", Toast.LENGTH_LONG).show();
                }
                else if(statusCode == 404){
                    Toast.makeText(getApplicationContext(), "404. Recurs no trobat", Toast.LENGTH_LONG).show();
                }
                else{
                    Toast.makeText(getApplicationContext(), "Dispositiu sense connexió o servidor no disponible", Toast.LENGTH_LONG).show();
                }
            }
        });
    }
}
