package com.biblioteca.jersey;

//@Autor: Andreu Querol Salsench
//Llicencia: Aquesta obra est subjecta a una llicncia de Reconeixement-NoComercial-SenseObraDerivada 3.0 Espanya de Creative Commons

import java.util.ArrayList;
import java.util.List;

import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;
 
//Classe que s'encarrega de convertir el resultat obtingut de la base de dades a format JSON
public class ConversorsJSON {
	
	//Metode que comprova si un string no es null o buida
	public static boolean stringNotNull (String text)
    {
    	if (text != null && text.trim().length() > 0)
    	{
    		return true;
    	}
    	else
    	{
    		return false;
    	}
    }
	
	//Metode que transforma un objecte llibre en JSON
    public static JSONObject llibreToJSON(Llibre llibre){
    	JSONObject ret = new JSONObject();
    		
    	try{
    		ret.put("id", llibre.getId());
    		ret.put("titol", llibre.getTitol());
    		ret.put("autor", llibre.getAutor());
    		ret.put("dataPublicacio", llibre.getDataPublicacio());
    		ret.put("imatge",llibre.getImatge());
    		ret.put("descripcio", llibre.getDescripcio());
    		ret.put("tema", llibre.getTema());
    		
    	}catch(JSONException e){
    		System.err.println("Error ConversorsJSON.LlibreToJSON: Error al crear el llibre en JSON");
    	}
    	
    	
    	return(ret);
    }
    
    //Metode que transforma un array de llibres a un array JSON
    public static JSONArray llistaLlibresToJSON(List<Llibre> llibres){
    	JSONArray llista = new JSONArray();
    	
    	for (int i =0; i<llibres.size();i++){
    		llista.put(llibreToJSON(llibres.get(i)));
    	}
		return llista;
    	
    }
    
    //metode que transforma la informacio d'un usuari a JSON
    public static JSONObject usuariToJSON(Usuari usuari){
    	JSONObject ret = new JSONObject();
    	
    	try{
    		ret.put("id", usuari.getId());
    		ret.put("nomUsuari", usuari.getNomUsuari());
    		
    	}catch(JSONException e){
    		System.err.println("Error ConversorsJSON.UsuariToJSON: Error al crear Usuari en JSON");
    	}
    	
    	return(ret);
    }
    
    //Metode que transforma la informacio sobre si una reserva s'ha dut a terme correctament a JSON
    public static JSONObject reservaOKToJSON (Boolean reservaOK, String dataTornada, String dataRecollida){
    	JSONObject ret = new JSONObject();
    	
    	try{
    		ret.put("reservaOK", reservaOK);
    		ret.put("dataTornada", dataTornada);
    		ret.put("dataRecollida", dataRecollida);
    	}catch(JSONException e){
    		System.err.println("Error ConversorsJSON.idUsuariToJSON: Error al crear la informacio de la reserva en JSON");
    	}
    	
    	return(ret);
    }
 
    //Metode que transforma la informacio de tots els usuaris que tenen un llibre reservat a JSON
    public static JSONObject infoReservaToJSON (ArrayList<String> llistaIdUsuaris, String dataReserva){
    	JSONObject ret = new JSONObject();
    	
    	try{
    		ret.put("dataReserva", dataReserva);
    		ret.put("llistaIdUsuaris", llistaIdUsuaris);
    		
    	}catch(JSONException e){
    		System.err.println("Error ConversorsJSON.idUsuariToJSON: Error al crear la informacio de la reserva en JSON");
    	}
    	
    	return(ret);
    }
    
    //Metode que transforma un llistat de reserves a JSON
    public static JSONArray llistaReservesToJSON (ArrayList <Reserva> llistaReserves){
    	JSONArray ret = new JSONArray();
    	
    	for (int i=0;i<llistaReserves.size();i++){
    		ret.put(reservaToJSON(llistaReserves.get(i)));
    	}
    	return ret;
    
    }
    
    //Metode que transforma una reserva a JSON
    public static JSONObject reservaToJSON (Reserva reserva){
    	JSONObject ret = new JSONObject();
    	
    	try{
    		ret.put("idLlibre", reserva.getIdLlibre());
    		ret.put("titol", reserva.getTitolLlibre());
    		ret.put("dataRecollida", reserva.getDataRecollida());
    		ret.put("dataTornada", reserva.getDataTornada());
    		ret.put("recordatori", reserva.getRecordatori());
    		ret.put("imatge", reserva.getImatge());
    		ret.put("numAmpliacions", reserva.getNumAmpliacions());
    		
    	}catch(JSONException e){
    		System.err.println("Error ConversorsJSON.ReservaToJSON: error al crear la informacio de la reserva en JSON");
    	}
    	
    	return ret;
    }
    
    //Metode que transforma el resultat d'una ampliacio de termini de prestec a JSON
    public static JSONObject ampliacioToJSON (Boolean resultatAmpliacio, int numeroAmpliacions){
    	JSONObject ret = new JSONObject();
    	
    	try{
    		ret.put("resultatAmpliacioOK", resultatAmpliacio);
    		ret.put("numeroAmpliacions", numeroAmpliacions);
    	}catch (JSONException e){
    		System.err.println("Error ConversorsJSON.ReservaToJSON: error al crear la informacio de l'ampliacio de prestec en JSON");
    	}
    	return ret;
    }
    
    //Metode que transforma el resultat d'activar/desactivar un recordatori de devolucio
    public static JSONObject activDesactivRecordatoriToJSON (int resultatRecordatori){
    	JSONObject ret = new JSONObject();
    	
    	try{
    		ret.put("resultatRecordatoriOK", resultatRecordatori);
    	}catch (JSONException e){
    		System.err.println("Error ConversorsJSON.ReservaToJSON: error al crear la informacio del recordatori de devolucio en JSON");
    	}
    	return ret;
    }
    
    
    //Metode que transforma un llistat de recordatoris a JSON
    public static JSONArray llistaRecordatorisToJSON (ArrayList <Recordatori> llistaRecordatoris){
    	JSONArray ret = new JSONArray();
    	
    	for (int i=0;i<llistaRecordatoris.size();i++){
    		ret.put(recordatoriToJSON(llistaRecordatoris.get(i)));
    	}
    	return ret;
    
    }
    
    //Metode que transforma una recordatori a JSON
    public static JSONObject recordatoriToJSON (Recordatori recordatori){
    	JSONObject ret = new JSONObject();
    	
    	try{
    		ret.put("data", recordatori.getData());
    		ret.put("titol", recordatori.getTitolLlibre());	
    	}catch(JSONException e){
    		System.err.println("Error ConversorsJSON.ReservaToJSON: error al crear la informacio del recordatori en JSON");
    	}
    	
    	return ret;
    }
    
    //Metode que transforma la resposta per indicar si una consulta ha sigut correcta o no en JSON 
    public static JSONObject resultatOKToJSON (Boolean ok){
    	JSONObject ret = new JSONObject();
    	try{
	    	if (ok==true){
	    		ret.put("resultatOK", true);
	    	}
	    	else
	    	{
	    		ret.put("resultatOK", false);
	    	}
    	}catch(JSONException e){
    		System.err.println("Error ConversorsJSON.resultatOKToJSON: error al crear la informacio del resultat de la consulta en JSON");
    	}
    	return ret;
    }
    
}